"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiService = void 0;
const url_1 = require("url");
const sha256_js_1 = require("@aws-crypto/sha256-js");
const signature_v4_1 = require("@aws-sdk/signature-v4");
const node_fetch_1 = __importDefault(require("node-fetch"));
const p_wait_for_1 = __importDefault(require("p-wait-for"));
const response_error_1 = require("../../../utils/errors/response-error");
const errors_1 = require("../../../utils/errors");
const POLLING_DEFAULT_INTERVAL_MS = 5000;
const POLLING_DEFAULT_TIMEOUT_MS = 5 * 60000;
function extractRegionFromApiGatewayEndpoint(endpointUrl) {
    const result = endpointUrl.match(/execute-api\.([^\.]*)/);
    if (result && result[1]) {
        return result[1];
    }
    return null;
}
function convertFetchHeaders(inputHeaders = {}) {
    const result = {};
    for (const [key, value] of Object.entries(inputHeaders)) {
        result[key] = value;
    }
    return result;
}
function convertURLSearchParamsToQueryBag(searchParams) {
    let queryBag = {};
    for (const [key, value] of searchParams.entries()) {
        queryBag[key] = value;
    }
    return queryBag;
}
class ApiService {
    constructor({ apiEndpoint, awsCredentialProvider }) {
        this.pollForDeploymentStatus = async (deploymentId, status, options = {}) => {
            const failureStatus = [
                'CREATE_FAILED',
                'DESTROY_FAILED',
            ];
            const { interval = POLLING_DEFAULT_INTERVAL_MS, timeout = POLLING_DEFAULT_TIMEOUT_MS, } = options;
            let result = null;
            await (0, p_wait_for_1.default)(async () => {
                const response = await this.getDeploymentById(deploymentId);
                if (response) {
                    if (failureStatus.indexOf(response.status) !== -1) {
                        throw new response_error_1.ResponseError({
                            status: 400,
                            code: 'DEPLOYMENT_CREATE_FAILED',
                        });
                    }
                    if (response.status === status) {
                        result = response;
                        return true;
                    }
                }
                return false;
            }, {
                interval,
                timeout,
                leadingCheck: false,
            });
            if (!result) {
                throw new Error('Could not get deployment status.');
            }
            return result;
        };
        this.deleteDeploymentById = (deploymentId) => {
            return this.fetchAWSSigV4(`/deployments/${deploymentId}`, {
                method: 'DELETE',
            });
        };
        this.apiEndpoint = apiEndpoint;
        this.awsCredentialProvider = awsCredentialProvider;
        const awsRegion = extractRegionFromApiGatewayEndpoint(apiEndpoint);
        if (!awsRegion) {
            throw new Error('API endpoint is in bad format. Could not extract region from it.');
        }
        this.awsRegion = awsRegion;
    }
    async fetchAWSSigV4(...fetchArgs) {
        const signature = new signature_v4_1.SignatureV4({
            region: this.awsRegion,
            service: 'execute-api',
            credentials: this.awsCredentialProvider,
            sha256: sha256_js_1.Sha256,
        });
        let requestUrl;
        if (typeof fetchArgs[0] === 'string') {
            requestUrl = fetchArgs[0];
        }
        else if ('href' in fetchArgs[0]) {
            requestUrl = fetchArgs[0].href;
        }
        else {
            requestUrl = fetchArgs[0].url;
        }
        const parsedUrl = new url_1.URL(requestUrl, this.apiEndpoint);
        try {
            const signedRequest = await signature.sign({
                hostname: parsedUrl.hostname,
                protocol: parsedUrl.protocol,
                path: parsedUrl.pathname,
                headers: convertFetchHeaders({
                    ...fetchArgs[1]?.headers,
                    host: parsedUrl.hostname,
                    accept: 'application/json',
                }),
                method: fetchArgs[1]?.method?.toUpperCase() ?? 'GET',
                query: convertURLSearchParamsToQueryBag(parsedUrl.searchParams),
                body: fetchArgs[1]?.body,
            });
            const response = await (0, node_fetch_1.default)(parsedUrl.href, {
                ...fetchArgs[1],
                headers: signedRequest.headers,
            });
            if (!response.ok) {
                throw await (0, response_error_1.createResponseError)(response);
            }
            if (response.headers.get('content-type') === 'application/json') {
                try {
                    return (await response.json());
                }
                catch (_ignoredError) {
                    return {};
                }
            }
            throw new Error('Invalid response from API');
        }
        catch (error) {
            if (error.name === 'CredentialsProviderError') {
                throw new errors_1.CredentialsError();
            }
            throw error;
        }
    }
    createAlias(requestBody) {
        return this.fetchAWSSigV4('/aliases', {
            method: 'POST',
            body: JSON.stringify(requestBody),
            headers: {
                'Content-Type': 'application/json',
            },
        });
    }
    deleteAlias(alias) {
        return this.fetchAWSSigV4(`/aliases/${alias}`, {
            method: 'DELETE',
        });
    }
    async listAliases(deploymentId) {
        const params = {
            deploymentId,
        };
        const query = new url_1.URLSearchParams(params).toString();
        const { items } = await this.fetchAWSSigV4(`/aliases?${query}`);
        return items;
    }
    createDeployment() {
        return this.fetchAWSSigV4('/deployments', {
            method: 'POST',
        });
    }
    async listDeployments() {
        const { items } = await this.fetchAWSSigV4('/deployments');
        return items;
    }
    getDeploymentById(deploymentId) {
        return this.fetchAWSSigV4(`/deployments/${deploymentId}`);
    }
}
exports.ApiService = ApiService;
