"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDeployCommand = void 0;
const path_1 = require("path");
const stream_1 = require("stream");
const chalk_1 = __importDefault(require("chalk"));
const clipboardy_1 = require("clipboardy");
const form_data_encoder_1 = require("form-data-encoder");
const formdata_node_1 = require("formdata-node");
const file_from_path_1 = require("formdata-node/file-from-path");
const node_fetch_1 = __importDefault(require("node-fetch"));
const client_1 = require("../../client");
const errors_1 = require("../../utils/errors");
async function deployCommand({ client, deploymentPackagePath = '.next-tf/deployment.zip', noClipboard, cwd, }) {
    const { apiService, output } = client;
    const internalDeploymentPackagePath = (0, path_1.resolve)(cwd, deploymentPackagePath);
    let deploymentId;
    output.spinner('Uploading deployment package');
    try {
        const deployment = await apiService.createDeployment();
        deploymentId = deployment.id;
        const uploadForm = new formdata_node_1.FormData();
        Object.entries(deployment.uploadAttributes).forEach(([key, value]) => {
            uploadForm.append(key, value);
        });
        uploadForm.append('file', await (0, file_from_path_1.fileFromPath)(internalDeploymentPackagePath));
        const encoder = new form_data_encoder_1.FormDataEncoder(uploadForm);
        const uploadResponse = await (0, node_fetch_1.default)(deployment.uploadUrl, {
            method: 'POST',
            headers: encoder.headers,
            body: stream_1.Readable.from(encoder),
        });
        if (uploadResponse.status !== 204) {
            console.log(uploadResponse.status);
            const parsedResponse = await uploadResponse.text();
            throw new Error(parsedResponse);
        }
        output.stopSpinner();
        output.success('Deployment package uploaded');
    }
    catch (error) {
        console.debug(error.toString());
        console.error('Could not upload deployment package.');
        return;
    }
    output.spinner('Waiting for deployment');
    try {
        const deploymentCreationResult = await apiService.pollForDeploymentStatus(deploymentId, 'FINISHED');
        output.stopSpinner();
        output.success('Deployment ready');
        if (deploymentCreationResult.deploymentAlias) {
            const previewUrl = `https://${deploymentCreationResult.deploymentAlias}`;
            let urlCopiedToClipboard = false;
            try {
                if (!noClipboard) {
                    (0, clipboardy_1.writeSync)(previewUrl);
                    urlCopiedToClipboard = true;
                }
            }
            catch (_ignoredError) { }
            output.log(`Available at: ${previewUrl}${urlCopiedToClipboard ? chalk_1.default.gray ` (copied to clipboard)` : ''} `);
        }
        else {
            output.log(`Deployment Id: ${deploymentCreationResult.id}`);
        }
    }
    catch (error) {
        if (error.code === 'DEPLOYMENT_CREATE_FAILED') {
            throw new errors_1.DeploymentCreateFailed();
        }
        throw error;
    }
}
const createDeployCommand = (0, client_1.withClient)('deploy', 'Deploy a project', (yargs) => {
    yargs.option('no-clipboard', {
        type: 'boolean',
        description: 'Do not copy the url to clipboard after a successful deployment',
    });
}, async ({ client, commandCwd, noClipboard }) => {
    await deployCommand({
        client,
        cwd: commandCwd,
        noClipboard: !!noClipboard,
    });
});
exports.createDeployCommand = createDeployCommand;
